// estim.js
// Nigel Whitfield, 2012
// change the eStimConnection URL to the address of your 2B
// it must be accessible from the system on which the browser is running
// Version 1.3

// change this line to the address or hostname of the system to which you want to connect 

var eStimConnection = 'http://localhost:7846/' ;

// set true to show debug info, or false otherwise

var debug = false ;

// you don't need to edit anything below this line

// for reference 
// build number first available as 16
// prior to that, will be undefined, and ;stopseq command not guaranteed to work
// from build 18: ;aplus, ;aminus, ;bplus, ;bminus, ;up, ;down available

var levelA = 0 ;
var levelB = 0 ;
var levelC = 50 ;
var levelD = 50 ;

var targetA, targetB, rampCount, timer ;

var reportErrors = true ;
var showAdvanced = false ;

var cLabels = new Array('Pulse speed','Pulse speed','Pulse feel','Pulse speed','Pulse speed','Rate change','Rate change','Pulse speed','Pulse speed','Throb range','Thrust range','Randomness','Step range','Jump range') ;

var dLabels = new Array('Pulse feel','Pulse feel','n/a','Pulse feel','Pulse feel','n/a','n/a','Pulse feel','Pulse feel','n/a','n/a','Pulse feel','Pulse feel','Pulse feel') ;

// this is what the modes are named in the favourites
var modeNames = new Array("Pulse","Bounce","Continuous","Asplit","Bsplit","Wave","Waterfall","Squeeze","Milk","Throb","Thrust","Random","Step","Training") ;

// for extended status

var remoteParams = new Array() ;
var remoteFavs = new Array() ;

// functions start here

function stimCommand(cmd) {

	stim = new XMLHttpRequest() ;
	
	stimDiv = document.getElementById('stimResults') ;
	
	stim.onreadystatechange = function() {
		if ( stim.readyState == 4 ) {
			if ( stim.responseText.substr(0,1) == ";" ) {
				// we got an extended response
				handleExtResponse(stim.responseText) ;
			} else {
				stimDiv.innerHTML = stim.responseText ;
				updateDisplay(stim.responseText) ;
			}
		}
	}
	
	stimURL = eStimConnection + cmd ;
	stim.open('GET',stimURL,true) ;
	stim.send(null) ;
	
	cmdDiv = document.getElementById('lastCmd') ;
	cmdDiv.innerHTML = cmd ;

}

function updateDisplay(estimResults) {
	
	var stimStatus = estimResults.split(":") ;
	
	if ( stimStatus.length != 9 && reportErrors ) {
		alert('Bad response received; server may be offline if this message repeats') ;
	} else {
	
		aSetting = document.getElementById('aLevel') ;
	 	  levelA = parseInt(stimStatus[1]/2) ;
	  	  aSetting.innerHTML = levelA ;
		bSetting = document.getElementById('bLevel') ;
	      levelB = parseInt(stimStatus[2]/2) ;
	      bSetting.innerHTML = levelB ;
	  
 		cSetting = document.getElementById('cLevel') ;
		  levelC = parseInt(stimStatus[3]/2) ;
	  	  cSetting.innerHTML = levelC ;

		dSetting = document.getElementById('dLevel') ;
	  	  levelD = parseInt(stimStatus[4]/2) ;
	  	  dSetting.innerHTML = levelD ;

	  
		power = stimStatus[6] ;
	 	  if ( power == 'H' ) {
		  	level = document.getElementById('powerH') ;
  	  	  } else {
		  	level = document.getElementById('powerL') ;
	  	  }
	  	  level.checked = true ;
	
		battery = document.getElementById('battLevel') ;
	  	  battery.innerHTML = parseInt(100*(stimStatus[0]-320)/422) ;
		program = document.getElementById('eMode') ;
	  	  program.selectedIndex = parseInt(stimStatus[5]) ;
		cLabel = document.getElementById('cLabel') ;
	  	  cLabel.innerHTML = cLabels[stimStatus[5]] ;
		dLabel = document.getElementById('dLabel') ;
	  	  dLabel.innerHTML = dLabels[stimStatus[5]] ;
		version = document.getElementById('firmware') ;
	  	  version.innerHTML = stimStatus[8] ;

	}
}

function setMode() {
	modeControl = document.getElementById('eMode') ;
	newMode = modeControl.selectedIndex ;
	
	stimCommand('M'+newMode)
}

function increaseA() {
	// in this and other functions, use the safer relative commands if available
	if ( remoteParams['buildno'] >= 18 ) { 
		stimCommand(';aplus')
	} else {
		aValue = levelA + 1 ;
		if (aValue > 100 ) {
			aValue = 100 ;
		}
		stimCommand("A" + aValue) ;
	}
}

function increaseB() {
	
	if ( remoteParams['buildno'] >= 18 ) {
		stimCommand(';bplus')
	} else {
		bValue = levelB + 1 ;
		if (bValue > 100 ) {
			bValue = 100 ;
		}
		stimCommand("B" + bValue) ;
	}
}

function increaseC() {
	cValue = levelC + 1 ;
	if (cValue > 100 ) {
		cValue = 100 ;
	}
	stimCommand("C" + cValue) ;
}

function increaseD() {
	dValue = levelD + 1 ;
	if (dValue > 100 ) {
		dValue = 100 ;
	}
	stimCommand("D" + dValue) ;
}

function decreaseA() {
	if ( remoteParams['buildno'] >= 18 ) {
		stimCommand(';aminus')
	} else {
		aValue = levelA - 1 ;
		if (aValue < 0 ) {
			aValue = 0 ;
		}
		stimCommand("A" + aValue) ;
	}
}

function decreaseB() {
	if ( remoteParams['buildno'] >= 18 ) {
		stimCommand(';bminus')
	} else {
		bValue = levelB - 1 ;
		if ( bValue < 0 ) {
			bValue = 0 ;
		}
		stimCommand("B" + bValue) ;
	}
}

function decreaseC() {
	cValue = levelC - 1 ;
	if (cValue < 2 ) {
		cValue = 2 ;
	}
	stimCommand("C" + cValue) ;
}

function decreaseD() {
	dValue = levelD - 1 ;
	if ( dValue < 2 ) {
		dValue = 2 ;
	}
	stimCommand("D" + dValue) ;
}

function gotoSettings() {
	// ramp the box up to specified settings, over a selected period of time
	
	reqTime = document.getElementById('gotoTime') ;
	
	if (reqTime.value < 30) {
		reqTime.value = 30 ;
	}
	timerSize = reqTime.value * 100 ;
	
	reportErrors = false ;
	
	// sort out the various settings
	gotoA = document.getElementById('gotoA') ;
	
	if ( gotoA.value == 0 ) {
		targetA = 0 ;
	} else {
		targetA = gotoA.value ;
	}
	
	gotoB = document.getElementById('gotoB') ;
	if ( gotoB.value == 0 ) {
		targetB = 0 ;
	} else {
		targetB = gotoB.value ;
	}
	
	gotoC = document.getElementById('gotoC') ;
	if ( gotoC.value == 0 ) {
		targetC = 50 ;
	} else {
		targetC = gotoC.value ;
	}
	
	gotoD = document.getElementById('gotoD') ;
	if ( gotoD.value == 0 ) {
		targetD = 50 ;
	} else {
		targetD = gotoD.value ;
	}
	
	
	// lock out the main controls
	mainCtrls = document.getElementById('mainControls') ;
	mainCtrls.style.display = "none" ;
	
	gotoCtrls = document.getElementById('gotoControls') ;
	gotoCtrls.style.display = "none" ;
	
	favCtrls = document.getElementById('favControls') ;
	favCtrls.style.display = "none" ;
	
	waitMsg = document.getElementById('waitBox') ;
	waitMsg.style.display = "block" ;
	
	// set the unchecked mode
	//stimCommand(";checks=off") ;
	
	// send the extended GOTO command
	
	stimCommand(";goto="+targetA+","+targetB+","+targetC+","+targetD+","+reqTime.value) ;
	
	
	// start the timer
	rampCount = 0 ;
	
	timer = window.setInterval(refresh,timerSize) ;
	
}

function refresh() {
	
	rampCount++ ;
	
	prog = document.getElementById('progressBox') ;
	prog.innerHTML = rampCount*10 + "%" ;
	
	
	if ( rampCount == 10 ) {
		window.clearInterval(timer)
				
		// re-enable controls
			
		rampCount = 0 ;
		
		//stimCommand(';checks=on') ;
		
		mainCtrls = document.getElementById('mainControls') ;
		mainCtrls.style.display = "block" ;
		
		favCtrls = document.getElementById('favControls') ;
		favCtrls.style.display = "block" ;
		
		gotoCtrls = document.getElementById('gotoControls') ;
		gotoCtrls.style.display = "none" ;
		
		advCtrls = document.getElementById('advancedCtrl') ;
		advCtrls.style.display = "none" ;
	
		waitMsg = document.getElementById('waitBox') ;
		waitMsg.style.display = "none" ;
		prog.innerHTML = "" ;
	
		stimCommand('S') ;
		
		reportErrors = true ;
		showAdvanced = false ;
	}
	
}


function allStop() {
	// send a stop command to the box
	// and make sure we're in a standard state
	window.clearInterval(timer) ;
	
	if ( remoteParams['buildno'] > 0 ) {
		stimCommand(';stopseq') ; // turn off any random modes and set output to zero
	} else {
		stimCommand('K') ;
	}
	//stimCommand(';checkson') ;
	reportErrors = true ;
	showAdvanced = false ;
		
	mainCtrls = document.getElementById('mainControls') ;
	mainCtrls.style.display = "block" ;
	
	favCtrls = document.getElementById('favControls') ;
	favCtrls.style.display = "block" ;
	
	advCtrls = document.getElementById('advancedCtrl') ;
	advCtrls.style.display = "none" ;
		
	gotoCtrls = document.getElementById('gotoControls') ;
	gotoCtrls.style.display = "none" ;
	
	waitMsg = document.getElementById('waitBox') ;
	waitMsg.style.display = "none" ;
	
	prog = document.getElementById('progressBox') ;
	prog.innerHTML = "" ;
	
}

function handleExtResponse(response) {
	// handle an extended response from the box
	// split the string into lines
	
	extLines = response.split("\r\n") ;
	
	for ( var c = 0 ; c < extLines.length ; c++ ) {
		if (extLines[c].substr(0,8) == ";STATUS:" ) {
			// this is a status response
			setting = extLines[c].substr(8).split("=") ;
			remoteParams[setting[0]] = setting[1] ;
		} else if (extLines[c].substr(0,5) == ";FAV:" ) {
			
			// it's a remote favourite
			favourite = extLines[c].substr(5).split(",") ;
			
			//alert("Processing favourite") ;
			remoteFav = { 'name' : favourite[0] ,
								'rise' : favourite[1] ,
								'run' : favourite[2] ,
								'fall' : favourite[3] ,
								'seq' : favourite[4] ,
								'mode' : favourite[5] ,
								'levelA' : favourite[6] ,
								'levelB' : favourite[7] ,
								'settingC' : favourite[8] ,
								'settingD' : favourite[9] ,
								'power' : favourite[10] }
			remoteFavs[remoteFavs.length] = remoteFav ;
		} 
	}
	
	if ( remoteFavs.length > 0 ) {
		buildFavs() ;
	} 
	
	// update the settings display
	updateSettings() ;
	
}

function resetFavs() {
	// get remote favourites, after resetting the local copy
	remoteFavs = Array() ;
	stimCommand(';GETFAVS') ;
	
}

function buildFavs() {
	// create a menu based on the favourites
	favMenu = document.getElementById("favList") ;
	favMenu.options.length = 0 ;
	
	for ( var c = 0 ; c < remoteFavs.length ; c++ ) {
		favMenu.options[favMenu.options.length] = new Option( remoteFavs[c].name , c ) ;
	}
	favMenu.setAttribute('onclick','') ;
	
	// if we can build a menu, we can get the system status, too
	// updateSettings() ;
}

function launchFav() {
	// send a GOTO command for the selected favourite
	
	favChosen = document.getElementById('favList').selectedIndex ;
	
	
	
	// hide main controls
	mainCtrls = document.getElementById('mainControls') ;
	mainCtrls.style.display = "none" ;
	
	gotoCtrls = document.getElementById('gotoControls') ;
	gotoCtrls.style.display = "none" ;
	
	favCtrls = document.getElementById('favControls') ;
	favCtrls.style.display = "none" ;
	
	advCtrls = document.getElementById('advancedCtrl') ;
	advCtrls.style.display = "none" ;
	
	waitMsg = document.getElementById('waitBox') ;
	waitMsg.style.display = "block" ;
	
	// we create the extended command, which has specifies rise time, mode and power level
	for ( var c = 0 ; c < modeNames.length ; c++ ) {
		if ( modeNames[c] == remoteFavs[favChosen].mode ) {
			mode = c ;
		}
	}
	
	stimCommand(";GOTO=" + remoteFavs[favChosen].levelA + "," + remoteFavs[favChosen].levelB + "," + remoteFavs[favChosen].settingC + "," +  remoteFavs[favChosen].settingD + "," + remoteFavs[favChosen].rise + "," + mode + "," + remoteFavs[favChosen].power)
	
	rampCount = 0 ;
	
	timer = window.setInterval(refresh,remoteFavs[favChosen].rise * 100) ;
	
}

function saveCurrent() {
	// save the current settings as a favourite
	// also reset the favourites list
	favMenu = document.getElementById("favList") ;
	favMenu.options.length = 0 ;
	favMenu.options[0] = new Option( "Click to reload", -1) ;
	favMenu.setAttribute('onclick','resetFavs()') ;
	
	stimCommand(';SAVE') ;
}

function toggleAdvanced() {
	// show hide advanced settings
	
	showAdvanced = ! showAdvanced ;
	gotoCtrls = document.getElementById('gotoControls') ;
	advCtrls = document.getElementById('advancedCtrl') ;
	
	if (showAdvanced) {
		stimCommand(';getstatus') ; // make sure we show the up to date settings
		gotoCtrls.style.display = "block" ;	
		advCtrls.style.display = "block" ;
	} else {
		gotoCtrls.style.display = "none" ;
		advCtrls.style.display = "none" ;
	}
}

function updateSettings() {
	
	// basic display of settings
	settings = document.getElementById('statusInfo') ;
	
	infoDisplay = 'countdown timer: ' + remoteParams['countdown'] + ' secs<br />random mode: ' + remoteParams['random'] + '<br />loop mode: ' + remoteParams['loop'] + '<br />sequence mode: ' + remoteParams['sequence'] + '<br />timeout: ' + remoteParams['timeout'] + ' secs' ;
	
	document.getElementById('delayFirst').value = remoteParams['delayvalue'] ;
	document.getElementById('loopRun').value = remoteParams['loopruntime'] ;
	document.getElementById('minDelay').value = remoteParams['mindelay'] ;
	document.getElementById('maxDelay').value = remoteParams['maxdelay'] ;
	document.getElementById('minRun').value = remoteParams['minrun'] ;
	document.getElementById('maxRun').value = remoteParams['maxrun'] ;
	document.getElementById('boostA').value = remoteParams['loopboosta'] ;
	document.getElementById('boostB').value = remoteParams['loopboostb'] ;
														  
	settings.innerHTML = infoDisplay ;
	
	// set the buttons for random mode according to the settings
	FDbutton = document.getElementById('buttonFD') ;
	DURbutton = document.getElementById('buttonDUR') ;
	INTbutton = document.getElementById('buttonINT') ;
	RUNbutton = document.getElementById('buttonRUN') ;
	BSTbutton = document.getElementById('buttonBST') ;
	
	if ( remoteParams['delayfirst'] == 'true' ) {
		FDbutton.className = 'buttonON' ;
	} else {
		FDbutton.className = 'buttonOFF' ;
	}
	
	if ( remoteParams['looptimeout'] == 'true' ) {
		DURbutton.className = 'buttonON' ;
	} else {
		DURbutton.className = 'buttonOFF' ;
	}
	
	if ( remoteParams['delaybetween'] == 'true' ) {
		INTbutton.className = 'buttonON' ;
	} else {
		INTbutton.className = 'buttonOFF' ;
	}
	
	if ( remoteParams['randomtimes'] == 'true' ) {
		RUNbutton.className = 'buttonON' ;
	} else {
		RUNbutton.className = 'buttonOFF' ;
	}
	
	if ( remoteParams['looppower'] == 'true' ) {
		BSTbutton.className = 'buttonON' ;
	} else {
		BSTbutton.className = 'buttonOFF' ;
	}
	
	document.getElementById('serverID').innerHTML = '<br />Server ' + remoteParams['version'] + ' (' + remoteParams['buildno'] + ')' ;
	
	
}

// these functions set options for the loop
// if the values are set to zero, we'll turn the option off
function setFirstDelay() {
	
	button = document.getElementById('buttonFD') ;
	
	if ( document.getElementById('delayFirst').value == 0 ) {
		stimCommand(';delayfirst=off') ;
		button.className = 'buttonOFF' ;
	} else {
		stimCommand(';delayfirst=' + document.getElementById('delayFirst').value) ;
		button.className = 'buttonON' ;
	}
}

function setLoop() {
	
	button = document.getElementById('buttonDUR') ;
	
	if ( document.getElementById('loopRun').value == 0 ) {
		stimCommand(';looptimeout=off') ; // run forever!
		button.className = 'buttonOFF' ;
	} else {
		stimCommand(';looptimeout=' + document.getElementById('loopRun').value) ;
		button.className = 'buttonON' ;
	}
}

function setDelay() {
	
	button = document.getElementById('buttonINT') ;

	if (( document.getElementById('minDelay').value == 0 ) && ( document.getElementById('maxDelay').value == 0 )) {
		stimCommand(';randomdelay=off') ;
		button.className = 'buttonOFF' ;
	} else {
		stimCommand(';randomdelay=' + document.getElementById('minDelay').value + ',' + document.getElementById('maxDelay').value) ;
		button.className = 'buttonON' ;
	}
}

function setRuntime() {
	
	button = document.getElementById('buttonRUN') ;

	if (( document.getElementById('minRun').value == 0 ) && ( document.getElementById('maxRun').value == 0 )) {
		stimCommand(';randomrun=off') ;
		button.className = 'buttonOFF' ;
	} else {
		stimCommand(';randomrun=' + document.getElementById('minRun').value + ',' + document.getElementById('maxRun').value) ;
		button.className = 'buttonON' ;
	}
}

function setBoost() {
	
	button = document.getElementById('buttonBST') ;

	if (( document.getElementById('boostA').value == 0 ) && ( document.getElementById('boostB').value == 0 )) {
		stimCommand(';loopboost=off') ;
		button.className = 'buttonOFF' ;
	} else {
		stimCommand(';loopboost=' + document.getElementById('boostA').value + ',' + document.getElementById('boostB').value) ;
		button.className = 'buttonON' ;
	}
}

function pageInit() {
	// set up the page with an initial server connection
	
	stimCommand(';getstatus')
	
	// show or hide the debug info
	debugInfo = document.getElementById('debug') ;
	if (debug) {
		debugInfo.style.display = 'block' ;
	} else {
		debugInfo.style.display = 'none' ;
	}	
}

function addButtons() {
	// display 'join' mode buttons, if appropriate
	
	if ( remoteParams['buildno'] >= 18 ) {
		
		// join commands available
		upButton = document.getElementById('joinUp') ;
		downButton = document.getElementById('joinDown') ;
		
		upButton.innerHTML = '<a class="bigButton" onclick="stimCommand(\';up\')">&uArr;</a>' ;
		downButton.innerHTML = '<a class="bigButton" onclick="stimCommand(\';down\')">&dArr;</a>' ;		
		
	}
}